import threading
from ._query_handler import QueryHandler
from ._query import Query
from ._query_response import QueryResponse
from ._exceptions import QueryAlreadyExistException
from ._exceptions import QueryHandlerDoesNotExistException
import threading


class QueryBus:
    def __init__(self):
        self._querys = {}
        self._semaphore = threading.Semaphore()

    def subscribe(self, query: Query, handler: QueryHandler):
        if query in self._querys:
            raise QueryAlreadyExistException()
        self._querys[query] = handler

    def publish(self, query: Query) -> QueryResponse:
        if query.__class__ not in self._querys:
            raise QueryHandlerDoesNotExistException()
        self._semaphore.acquire()
        response: QueryResponse = self._querys[query.__class__].handle(query)
        self._semaphore.release()
        return response
        


class CommandBusException(Exception):
    pass


class CommandAlreadyExistException(CommandBusException):
    pass


class CommandHandlerDoesNotExistException(CommandBusException):
    pass

class QueryBusException(Exception):
    pass


class QueryAlreadyExistException(QueryBusException):
    pass


class QueryHandlerDoesNotExistException(QueryBusException):
    pass



from distutils.core import setup

setup(
  name = 'plato_cqrs',
  packages = ['plato_cqrs'],
  version = '0.1.1',
  license='GPL-3',
  description = 'Simple CQRS implementation',
  author = 'Pepe Márquez Doblas',
  author_email = 'pepemarquezof@gmail.com',
  url = 'https://github.com/IronSenior/PythonCommandQuery',
  download_url = 'https://github.com/IronSenior/PythonCommandQuery/archive/refs/tags/0.1.tar.gz',
  keywords = ['CQRS', 'COMMAND', 'QUERY'],
  install_requires=[],
  classifiers=[
    'Development Status :: 4 - Beta',
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
    'Programming Language :: Python :: 3.9'
  ],
)


from ._exceptions import *
from ._command import Command
from ._command_bus import CommandBus
from ._command_handler import CommandHandler
from ._exceptions import *
from ._query import Query
from ._query_bus import QueryBus
from ._query_handler import QueryHandler
from ._query_response import QueryResponse


from ._command_handler import CommandHandler
from ._command import Command
from ._exceptions import CommandAlreadyExistException
from ._exceptions import CommandHandlerDoesNotExistException
import threading


class CommandBus:
    def __init__(self):
        self._commands = {}
        self._semaphore = threading.Semaphore()

    def subscribe(self, cmd: type, handler: CommandHandler):
        if cmd in self._commands:
            raise CommandAlreadyExistException()
        self._commands[cmd] = handler

    def publish(self, cmd: Command):
        if cmd.__class__ not in self._commands:
            raise CommandHandlerDoesNotExistException()
        self._semaphore.acquire()
        self._commands[cmd.__class__].handle(cmd)
        self._semaphore.release()
